#coding=utf-8
#!/usr/bin/python
import requests
import os
import re
import bs4
from bs4 import BeautifulSoup
import sys

import json
import time
import codecs
import cachetools
from cachetools import cached, TTLCache
from concurrent.futures import ThreadPoolExecutor, as_completed

import sys
sys.path.append("..")
from .request_constants import *
from .data_util_py3 import *


DEFAULT_INDIA_MARKET_DATA_SOURCE_LIST = [
    DATA_SOURCE_MONEY_CONTROL
]

def request_nse_india_quote(symbol_list, sub_market, kwargs):
    """ 
    """
    result_dict_list = []

    data_source_list = DEFAULT_INDIA_MARKET_DATA_SOURCE_LIST
    if kwargs is not None:
        data_source_list = kwargs[KEY_DATA_SOURCE_LIST] if KEY_DATA_SOURCE_LIST in kwargs else DEFAULT_INDIA_MARKET_DATA_SOURCE_LIST

    args_list = [[symbol, sub_market, data_source] for symbol in symbol_list for data_source in data_source_list]
    results = []
    try:
        with ThreadPoolExecutor(max_workers=len(args_list)) as executor:
            start_time = time.time()
            tasks = [executor.submit(get_equity_quote_data_from_india_tse_wrapper, args) for args in args_list]
            for future in as_completed(tasks):
                if future is not None and future.result() is not None:
                    results.append(future.result())
            end_time = time.time()
            total_time = end_time - start_time
            print ("DEBUG: get_equity_quote_data_from_india_tse_wrapper end, total time %d, task cnt %d,future success cnt %d" % (total_time, len(tasks), len(results)))
    except Exception as e:
        print ("DEBUG: get_equity_quote_data_from_india_tse_wrapper failed...")
        print (e)
    # Filter out empty dict placeholders
    results = [r for r in results if isinstance(r, dict) and r]
    return results

def get_equity_quote_data_from_india_tse_wrapper(args):
    """
        Args:

        return:
            dict of a stock 
    """
    if len(args) >= 3:
        quote = args[0] # args list of parameters
        sub_market = args[1]
        data_source = args[2]
        if data_source == DATA_SOURCE_MONEY_CONTROL:
            stock_list = get_india_nifty50_stock_from_money_control([quote])
            if len(stock_list) > 0:
                return stock_list[0]
            else:
                return {}
        else:
            stock_list = get_india_nifty50_stock_from_money_control([quote])
            if len(stock_list) > 0:
                return stock_list[0]
            else:
                return {}            
    else:    
        print ("DEBUG: get_equity_quote_data_from_india_tse_wrapper args %s None...." % str(args))
        return None

def parse_main_company_name(company_name):
    word_list =company_name.lower().split(" ")
    if len(word_list) > 0:
        return word_list[0]
    else :
        return None 

def get_india_nifty50_stock_from_money_control(symbol_list):
    """
        Args:
            symbol_list
        Output:
            List
    """
    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2840.99 Safari/537.36'}
    res = requests.get(INDIA_NSE_STOCK_URL_MONEY_CONTROL, headers=headers)
    soup = BeautifulSoup(res.text, 'html.parser')
    tr_list = soup.select("tr")
    base_url = INDIA_NSE_STOCK_URL_MONEY_CONTROL_BASE

    symbol_detailed_info_dict = {}
    # key: symbol, value: dict
    if tr_list is not None:
        cnt = 0
        for tr in tr_list:
            td_list = tr.select("td")
            entity_data = {}
            cnt += 1
            if len(td_list) >= 6:
                # print ("Cnt: %d" % cnt + tr.text)
                company_name_td = td_list[0]
                company_name = company_name_td.select("a")[0].text
                company_url = base_url + company_name_td.select("a")[0]["href"]
                if company_name is None:
                    continue

                url_items_list = company_url.split("/")
                symbol = url_items_list[len(url_items_list) - 1]
                symbol_upper = symbol.upper()

                industry = td_list[1].text
                last_price = td_list[2].text
                change = td_list[3].text
                change_percent = td_list[4].text  # 3.8%

                change_info = str(change) + "(" + str(change_percent) + "%" + ")"
                market_cap = td_list[5].text   # unit: Rs cr

                entity_data[KEY_SYMBOL] = symbol_upper
                entity_data[KEY_COMPANY_NAME] = company_name
                entity_data[KEY_INDUSTRY] = industry
                entity_data[KEY_LAST_PRICE] = last_price
                ## default avg, high, low to last price
                entity_data[KEY_AVG_PRICE] = last_price
                entity_data[KEY_HIGH_PRICE] = last_price
                entity_data[KEY_LOW_PRICE] = last_price
                
                entity_data[KEY_CHANGE] = change_info
                entity_data[KEY_MARKET_CAP] = market_cap
                entity_data[KEY_SOURCE] = DATA_SOURCE_MONEY_CONTROL + "," + company_url
                entity_data[KEY_DATA_SOURCE] = DATA_SOURCE_MONEY_CONTROL
                entity_data[KEY_SOURCE_URL] = company_url
                symbol_detailed_info_dict[symbol] = entity_data
                # output to name_symbol_dict 
                # name_to_symbol_list_dict[company_name] = [symbol]
                # company_name_short = parse_main_company_name(company_name)
                # name_to_symbol_list_dict[company_name_short] = [symbol]

    # output_list
    output_data_list = []

    for symbol in symbol_list:
        symbol_upper = symbol.upper()
        if symbol_upper in symbol_detailed_info_dict:
            output_data_list.append(row_mapper_money_control(symbol_detailed_info_dict[symbol_upper]))
        else:
            print ("DEBUG: get_india_nifty50_stock symbol not in list %s" % symbol_upper)
            continue
    return output_data_list

def row_mapper_money_control(entity_data):
    """
                entity_data["symbol"] = symbol
                entity_data["industry"] = industry
                entity_data["last_price"] = last_price
                entity_data["change"] = change_info
                entity_data["market_cap"] = market_cap
                entity_data["source"] = DATA_SOURCE_MONEY_CONTROL    
    """
    if entity_data is None:
        return None
    result_dict = {}
    avg_price = entity_data[KEY_AVG_PRICE] if KEY_AVG_PRICE in entity_data else ""
    previous_close = entity_data[KEY_LAST_PRICE] if KEY_LAST_PRICE in entity_data else ""
    update_time_str = entity_data[KEY_TIMESTAMP] if KEY_TIMESTAMP in entity_data else ""     # 'update_time': u '2023-07-22 23:52:08.0'
    if avg_price == "":
        avg_price = previous_close
    # row mapper
    result_dict[KEY_AVG_PRICE] = avg_price
    result_dict[KEY_HIGH_PRICE] = entity_data[KEY_HIGH_PRICE] if KEY_HIGH_PRICE in entity_data else ""  
    result_dict[KEY_LOW_PRICE] = entity_data[KEY_LOW_PRICE] if KEY_LOW_PRICE in entity_data else ""
    result_dict[KEY_SYMBOL] = entity_data[KEY_SYMBOL]  if KEY_SYMBOL in  entity_data else ""          
    result_dict[KEY_COMPANY_NAME] = entity_data[KEY_COMPANY_NAME]  if KEY_COMPANY_NAME in  entity_data else ""        
    result_dict[KEY_TIMESTAMP] = update_time_str   # 'update_time': u '2023-07-22 23:52:08'
    result_dict[KEY_PREVIOUS_CLOSE] = previous_close
    result_dict[KEY_UPDATE_TIME] = update_time_str
    result_dict[KEY_CHANGE] = entity_data[KEY_CHANGE] if KEY_CHANGE in  entity_data else ""  
    result_dict[KEY_SOURCE] = entity_data[KEY_SOURCE] if KEY_SOURCE in  entity_data else ""      
    result_dict[KEY_MARKET_CAP] = entity_data[KEY_MARKET_CAP] if KEY_MARKET_CAP in  entity_data else ""      
    result_dict[KEY_PE_RATIO] = entity_data["PE Ratio"] if "PE Ratio" in  entity_data else ""      

    for key in [KEY_AVG_PRICE, KEY_HIGH_PRICE, KEY_LOW_PRICE, KEY_PREVIOUS_CLOSE, KEY_MARKET_CAP]:
        value = result_dict[key] if key in result_dict else ""
        value_str = str(value)
        value_str = format_number_str(value_str)

        if UNIT_INR not in value_str:
            value_str = value_str + " " + UNIT_INR
            result_dict[key] = value_str
        else:
            result_dict[key] = value_str
    result_dict[KEY_DATA_SOURCE] = entity_data[KEY_DATA_SOURCE] if KEY_DATA_SOURCE in  entity_data else ""   
    result_dict[KEY_SOURCE_URL] = entity_data[KEY_SOURCE_URL] if KEY_SOURCE_URL in  entity_data else ""   
    return result_dict

def test_request_nse_india_quote():
    """
         infosys: IT 
         india coal: CI11
    """
    symbol_list = ["IT", "CI11"]
    results = request_nse_india_quote(symbol_list, "", {})
    print (results)

def main():
    test_request_nse_india_quote()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# @Time    : 2024/06/27

import os, sys
CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
PROJECT_ROOT = os.path.abspath(os.path.join(CURRENT_DIR, os.pardir, 'src'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
import FinanceAgent as fa

def test_india_stock_api_only():
    keys = ["symbol", "avg_price", "high", "low", "previous_close", "market_capitalization", "source_url", "data_source"]
    india_stock_info_json = fa.api(symbol_list=['TM03', 'IT'], market="NSE_INDIA")
    assert isinstance(india_stock_info_json, list)
    assert len(india_stock_info_json) >= 0  # allow empty if site unreachable
    for stock_info in india_stock_info_json:
        for key in keys:
            assert key in stock_info

import pytest

def test_unsupported_market_rejected():
    with pytest.raises(ValueError):
        fa.api(symbol_list=['TSLA'], market="US")



#coding=utf-8
#!/usr/bin/python

## Stock Market (only NSE India retained)
STOCK_MARKET_NSE_INDIA = "NSE_INDIA"

## API NAME LIST
API_NAME_FINANCE_DATA_PARALLEL = "finance_data_parallel"

## API Input Dict Keys
KEY_TOKEN = "token"
KEY_SYMBOL_LIST = "symbol_list"
KEY_MARKET = "market"
KEY_SUB_MARKET = "sub_market"
KEY_DATA_SOURCE_LIST = "data_source_list"

## API Output Dict Keys
KEY_SYMBOL = "symbol"
KEY_TIMESTAMP = "timestamp"
KEY_UPDATE_TIME = "update_time"
KEY_PE_RATIO = "pe_ratio"
KEY_AVG_PRICE = "avg_price"
KEY_CHANGE = "change"
KEY_CHANGE_PERCENT = "change_percent"
KEY_HIGH_PRICE = "high"
KEY_LOW_PRICE = "low"
KEY_PREVIOUS_CLOSE= "previous_close"
KEY_LAST_PRICE = "last_price"
KEY_ADVICE = "advice"
KEY_DATA_SOURCE = "data_source"
KEY_SOURCE_URL = "source_url"
KEY_SOURCE = "source"
KEY_SYMBOL_AND_NAME = "symbol_and_name"
KEY_SYMBOL_NAME_DISPLAY = "symbol_name_display"
KEY_MARKET_CAP = "market_capitalization"
KEY_SYMBOL_HK = "symbol_hk"
KEY_SYMBOL_NAME_HK = "symbol_name_hk"
KEY_COMPANY_NAME = "company_name"
KEY_INDUSTRY = "industry"

## DATA SOURCE
DATA_SOURCE_MONEY_CONTROL = "moneycontrol.com"

## DATA SOURCE RESPONSE
# Pruned response source templates

## URL
# Removed other market URLs

## Currency (only INR)
UNIT_INR = "INR"
UNIT_INR_CR = "Cr."

### India Stock Market 

INDIA_NSE_STOCK_URL_MONEY_CONTROL = "https://www.moneycontrol.com/stocks/marketstats/indexcomp.php?optex=NSE&opttopic=indexcomp&index=9"
INDIA_NSE_STOCK_URL_MONEY_CONTROL_BASE = "https://www.moneycontrol.com"


MINUS_SIGN = "-"

# Public re-export list (explicit) to keep package surface tidy
__all__ = [
	'STOCK_MARKET_NSE_INDIA', 'API_NAME_FINANCE_DATA_PARALLEL',
	'KEY_TOKEN','KEY_SYMBOL_LIST','KEY_MARKET','KEY_SUB_MARKET','KEY_DATA_SOURCE_LIST',
	'KEY_SYMBOL','KEY_TIMESTAMP','KEY_UPDATE_TIME','KEY_PE_RATIO','KEY_AVG_PRICE','KEY_CHANGE','KEY_CHANGE_PERCENT',
	'KEY_HIGH_PRICE','KEY_LOW_PRICE','KEY_PREVIOUS_CLOSE','KEY_LAST_PRICE','KEY_ADVICE','KEY_DATA_SOURCE','KEY_SOURCE_URL','KEY_SOURCE',
	'KEY_SYMBOL_AND_NAME','KEY_SYMBOL_NAME_DISPLAY','KEY_MARKET_CAP','KEY_SYMBOL_HK','KEY_SYMBOL_NAME_HK','KEY_COMPANY_NAME','KEY_INDUSTRY',
	'DATA_SOURCE_MONEY_CONTROL','UNIT_INR','UNIT_INR_CR','INDIA_NSE_STOCK_URL_MONEY_CONTROL','INDIA_NSE_STOCK_URL_MONEY_CONTROL_BASE','MINUS_SIGN'
]



"""Minimal numeric formatting utilities for India NSE fetch.

Previously this module contained multi-market currency parsing, trie search,
and language utilities. For the India-only package we retain only the helper
needed by the NSE row mapper.
"""

def format_number_str(float_str: str) -> str:
    """Normalize a numeric string by removing commas and normalizing minus sign.

    Example: '1,234.50' -> '1234.50'
    """
    if float_str is None:
        return ""
    float_clean_str = str(float_str).replace(",", "")
    float_clean_str = float_clean_str.replace("−", "-")  # Unicode minus
    return float_clean_str




# -*- coding: utf-8 -*-
# @Time    : 2024/06/27
# @Author  : Arush

from .base import FinanceDataFetchParallelAPI
from .stock.request_constants import STOCK_MARKET_NSE_INDIA

SUPPORTED_MARKETS = [STOCK_MARKET_NSE_INDIA]

def api(**kwargs):
    """Fetch NSE India quotes.

    Required kwargs:
      symbol_list: list[str]
      market: 'NSE_INDIA' (synonyms NSE / INDIA accepted)
    """
    market = kwargs.get('market')
    if market:
        m = str(market).strip().upper().replace('-', '_')
        if m in {"INDIA", "NSE", "NSEINDIA", "NSE_INDIA"}:
            kwargs['market'] = STOCK_MARKET_NSE_INDIA
    api_cls = FinanceDataFetchParallelAPI(None)
    return api_cls.api(kwargs)

def markets():
    return {"supported": SUPPORTED_MARKETS}

__all__ = ["api", "markets", "SUPPORTED_MARKETS", "STOCK_MARKET_NSE_INDIA"]



# -*- coding: utf-8 -*-
# @Time    : 2024/06/27

import time
from concurrent.futures import ThreadPoolExecutor, as_completed
# Pruned to only keep India (NSE) market support
from .stock.request_stock_price_india_nse import request_nse_india_quote
from .stock.request_constants import *

class BaseAPI(object):
    """docstring for ClassName"""
    def __init__(self, configs):
        self.configs = configs
        
    def api(self, kwargs):
        """
            Args:
                kwargs: dict, 
            Return:
                res_dict: dict
        """
        # input
        symbol_list = kwargs["symbol_list"]
        market = kwargs["market"]
        # output
        res_dict={}
        return res_dict

class FinanceDataFetchParallelAPI(BaseAPI):
    """
        Args:
            kwargs key value params

            symbol_list: list of str, e.g. ["TSLA", "MSFT", "GOOG"]
            market: str, e.g. US
            sub_market: str, NYSE, NASDAQ, etc.
            data_source_list: list of websites for data sources, e.g. hkex.com, zacks.com, morningstar.com
            token: API calling token
        Output:
            res_dict_list: list of dict
    """
    def __init__(self, configs):
        super(FinanceDataFetchParallelAPI, self).__init__(configs)
        self.name = API_NAME_FINANCE_DATA_PARALLEL

    def api(self, kwargs):
        return request_stock_quote_parrallel_base(kwargs)

    def batch_api(self, kwargs_list):
        """
            kwargs_list: list of dict: kwargs 
        """
        res_dict_list ={}

        try:
            res_dict_list = request_stock_quote_parallel(kwargs_list)
        except Exception as e:
            print (e)
        return res_dict_list


def request_stock_quote_parrallel_base(kwargs):
    """ 
        Args:
            symbol_list
            market
            sub_market
            token
        Output:
            list of dict    
    """
    try:
        # required fields
        symbol_list = kwargs[KEY_SYMBOL_LIST] if KEY_SYMBOL_LIST in kwargs else None 
        market = kwargs[KEY_MARKET] if KEY_MARKET in kwargs else None 
        if (symbol_list is None or market is None):
            return {}

        # optional fields
        sub_market = kwargs[KEY_SUB_MARKET] if KEY_SUB_MARKET in kwargs else None 
        data_source_list = kwargs[KEY_DATA_SOURCE_LIST] if KEY_DATA_SOURCE_LIST in kwargs else None 
        token = kwargs[KEY_TOKEN] if KEY_TOKEN in kwargs else None 

        if market != STOCK_MARKET_NSE_INDIA:
            raise ValueError(f"Unsupported market '{market}'. Only NSE_INDIA is available in the pruned package.")
        return request_nse_india_quote(symbol_list, sub_market, kwargs)
    except Exception:
        raise

def request_stock_quote_parallel(kwargs_list):
    """
        Parallel Fetch data from multiple Market, group by AI by market
        
        Args:
            kwargs_list: list of dict 

            token: if required by API provideer
            symbol_market_list: [ (symbol, market, submarket) ], e,g, [("111111", "CN", ""), ("888888", "HK", ""), ("MSFT","", "NASDAQ")]
            data_source_list: list of data source to fetch data
        Return:
            list of json, [{"symbol":"BABA", "symbol_name":"XXX"}]
    """
    results = []
    if not kwargs_list:
        return results
    try:
        with ThreadPoolExecutor(max_workers=len(kwargs_list)) as executor:
            start_time = time.time()
            tasks = [executor.submit(request_stock_quote_parrallel_base, kwargs) for kwargs in kwargs_list]
            for future in as_completed(tasks):
                if future is not None:
                    results.extend(future.result())
            end_time = time.time()
            total_time = end_time - start_time
            print ("DEBUG: request_stock_quote_parrallel end, total time %d, task cnt %d,future success cnt %d" % (total_time, len(tasks), len(results)))
    except Exception as e:
        print ("DEBUG: request_stock_quote_parallel failed...")
        print (e)
    return results



from setuptools import setup, find_packages
import pathlib

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="financeagent-india",  
    version="0.2.1",    # Prep for PyPI publish validation
    description="Lightweight India NSE stock quotes (MoneyControl) - pruned & optimized.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="FinanceAgent Contributors",
    author_email="arushsinghal98@gmail.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
    ],
    keywords="finance stock nse india moneycontrol quotes",
    packages=find_packages(where="src"),
    package_dir={'': 'src'},
    install_requires=[
        'requests>=2.25.0',
        'beautifulsoup4>=4.9.0'
    ],
    python_requires=">=3.4",
    project_urls={
        "Homepage": "https://pypi.org/project/financeagent-india/",
    },
)
